﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.IO;
using System.IO.IsolatedStorage;
using System.Linq;
using System.Net;
using System.Runtime.Serialization;
using System.ServiceModel.Syndication;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Xml;
using System.Xml.Serialization;

namespace SilverlightSyndication
{
    public partial class Reader : UserControl
    {
        #region Fields

        private List<SyndicationItem> _feedItems;
        private ObservableCollection<SyndicationFeed> _feeds = new ObservableCollection<SyndicationFeed>();
        private const string FEED_FILENAME = "FeedList.dat";
        private const string FEED_DATA = "FeedListData";
        private const string FEED_DELIMITER = "|";

        #endregion

        #region Constructor(s)

        public Reader()
        {
            InitializeComponent();
            btnAdd.Click += new RoutedEventHandler(btnAdd_Click);
            btnRefresh.Click += new RoutedEventHandler(btnRefresh_Click);
            feedDataGrid.KeyUp += new KeyEventHandler(lstFeeds_KeyUp);

            _feeds.CollectionChanged += new System.Collections.Specialized.NotifyCollectionChangedEventHandler(Feeds_CollectionChanged);

            //TODO: For testing only
            //ClearStorage_UsingFile();

            // Get the feeds
            LoadFeedsFromStorage_UsingSettings();

            feedDataGrid.DataContext = _feeds;
            feedItemsGridLayout.DataContext = _feedItems;
        }

        private void btnRefresh_Click(object sender, RoutedEventArgs e)
        {
            ReBindAggregatedItems();
        }

        private byte[] Serialize(object o)
        {
            DataContractSerializer serializer = new DataContractSerializer(o.GetType());
            MemoryStream memoryStream = new MemoryStream();
            serializer.WriteObject(memoryStream, o);
            return memoryStream.ToArray();
        }

        private object Deserialize(Type type, byte[] objectBytes)
        {
            DataContractSerializer serializer = new DataContractSerializer(type);
            MemoryStream memoryStream = new MemoryStream(objectBytes);
            return serializer.ReadObject(memoryStream);
        }

        //Works cool for FeedBurner
        //private string PackUris()
        //{
        //    var query = from f in _feeds
        //                from l in f.Links
        //                where l.RelationshipType == "self"
        //                select l.GetAbsoluteUri().AbsoluteUri;
        //    string data = string.Empty;
        //    foreach (var url in query)
        //        data += url + FEED_DELIMITER;
        //    return data;
        //}

        private void ClearStorage_UsingSettings()
        {
            if (IsolatedStorageSettings.ApplicationSettings.Contains(FEED_DATA))
                IsolatedStorageSettings.ApplicationSettings[FEED_DATA] = string.Empty;
            else
                IsolatedStorageSettings.ApplicationSettings.Add(FEED_DATA, string.Empty);
        }

        private void ClearStorage_UsingFile()
        {
            try
            {
                var isoStore = IsolatedStorageFile.GetUserStoreForApplication();
                isoStore.DeleteFile(FEED_FILENAME);
            }
            catch
            {
                // ignore it
            }
        }

        private void SaveFeedsToStorage_UsingSettings()
        {
            string data = GetFeedsFromStorage_UsingSettings() + FEED_DELIMITER + txtAddress.Text;

            if (IsolatedStorageSettings.ApplicationSettings.Contains(FEED_DATA))
                IsolatedStorageSettings.ApplicationSettings[FEED_DATA] = data;
            else
                IsolatedStorageSettings.ApplicationSettings.Add(FEED_DATA, data);
        }

        private void SaveFeedsToStorage_UsingFile()
        {
            using (var isoStore = IsolatedStorageFile.GetUserStoreForApplication())
            {
                List<string> data = GetFeedsFromStorage_UsingFile();

                if (data == null)
                    if (txtAddress.Text.Length == 0)
                        return;
                    else
                        data = new List<string>();

                using (var isoStoreFileStream = new IsolatedStorageFileStream(FEED_FILENAME, FileMode.Create, isoStore))
                {
                    data.Add(txtAddress.Text);

                    byte[] bytes = Serialize(data);
                    isoStoreFileStream.Write(bytes, 0, bytes.Length);
                }
            }
        }

        private string GetFeedsFromStorage_UsingSettings()
        {
            string data = string.Empty;
            if (IsolatedStorageSettings.ApplicationSettings.Contains(FEED_DATA))
                data = IsolatedStorageSettings.ApplicationSettings[FEED_DATA].ToString();
            return data;
        }

        private List<string> GetFeedsFromStorage_UsingFile()
        {
            byte[] feedBytes;
            var ms = new MemoryStream();
            using (var isoStore = IsolatedStorageFile.GetUserStoreForApplication())
            {
                //If the file does not exist, get out.
                if (!isoStore.FileExists(FEED_FILENAME)) return null;

                using (var stream = isoStore.OpenFile(FEED_FILENAME, FileMode.Open, FileAccess.Read))
                {
                    while (true)
                    {
                        byte[] tempBytes = new byte[1024];
                        int read = stream.Read(tempBytes, 0, tempBytes.Length);
                        if (read <= 0)
                        {
                            //feedBytes = ms.ToArray();
                            break;
                        }
                        ms.Write(tempBytes, 0, read);
                    }
                }
                feedBytes = ms.ToArray();
                List<string> feedList = Deserialize(typeof(List<string>), feedBytes) as List<string>;
                return feedList;
            }
        }

        private void LoadFeedsFromStorage_UsingSettings()
        {
            string data = GetFeedsFromStorage_UsingSettings();

            string[] feedList = data.Split(new string[1] { FEED_DELIMITER }, StringSplitOptions.RemoveEmptyEntries);
            foreach (var address in feedList)
                LoadFeed(new Uri(address));
        }

        private void LoadFeedsFromStorage_UsingFile()
        {
            var feedList = GetFeedsFromStorage_UsingFile();
            foreach (var address in feedList)
            {
                Uri feedUri;
                Uri.TryCreate(address, UriKind.Absolute, out feedUri);
                if (feedUri != null)
                    LoadFeed(feedUri);
            }
        }

        #endregion

        #region Event Handlers

        private void lstFeeds_KeyUp(object sender, KeyEventArgs e)
        {
            SyndicationFeed feed = feedDataGrid.SelectedItem as SyndicationFeed;
            if (feed == null)
                return;

            if (e.Key == Key.Delete)
            {
                _feeds.Clear(); // For now, let's clear them all since Yahoo Pipes does not expose the link as needed.
                ClearStorage_UsingFile();

                //_feeds.Remove(feed);
                ReBindAggregatedItems();
            }
        }

        private void btnAdd_Click(object sender, RoutedEventArgs e)
        {
            Uri feedUri;
            Uri.TryCreate(txtAddress.Text, UriKind.Absolute, out feedUri);
            if (feedUri == null)
                return;

            LoadFeed(feedUri);
        }

        public void LoadFeed(Uri feedUri)
        {
            WebClient request = new WebClient();
            request.DownloadStringCompleted += AddFeedCompleted;
            request.DownloadStringAsync(feedUri);
        }

        private void AddFeedCompleted(object sender, DownloadStringCompletedEventArgs e)
        {
            if (e.Error != null)
                return;

            string xml = e.Result;
            if (xml.Length == 0)
                return;

            StringReader stringReader = new StringReader(xml);
            XmlReader reader = XmlReader.Create(stringReader);
            SyndicationFeed feed = SyndicationFeed.Load(reader);

            if (_feeds.Where(f => f.Title.Text == feed.Title.Text).ToList().Count > 0)
                return;

            _feeds.Add(feed); // This also saves the feeds to isolated storage
            ReBindAggregatedItems();

            txtAddress.Text = string.Empty;
        }

        private void Feeds_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            SaveFeedsToStorage_UsingSettings();
        }

        #endregion

        #region Private Helpers

        private void ReBindAggregatedItems()
        {
            //Read the feed items and bind them to the lower list

            var query = from f in _feeds
                        from i in f.Items
                        orderby i.PublishDate descending
                        select new SyndicationItemExtra { FeedTitle = f.Title.Text, Item = i };

            var items = query.ToList();

            feedItemsGridLayout.DataContext = items;
        }

        #endregion
    }
}
